﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;
using FarPoint.Win.Spread.UndoRedo;
using System.Diagnostics;

namespace PowerOffice.SmartClient.DocumentCenter.Views.PowerFormsDesignerView.Actions
{
    /// <summary>
    /// Representerer en handling som kan angres.
    /// Denne handlingen utføres på et utvalg av celler.
    /// </summary>
    /// <typeparam name="OriginalDataType"></typeparam>
    abstract class CellSelectionAction<OriginalDataType>: SheetAction
    {
        /// <summary>
        /// Konstruerer et ny handlingsobjekt.
        /// </summary>
        /// <param name="sheet"></param>
        /// <param name="selection"></param>
        protected CellSelectionAction(SheetView sheet)
            : base(sheet)
        {
            this.Selection = new CellSelection(sheet);
            this.OriginalStates = new OriginalDataType[this.Selection.RowCount, this.Selection.ColumnCount];
            this.SaveUndoState();
        }
        
        /// <summary>
        /// Henter handlingens utvalg.
        /// </summary>
        public CellSelection Selection { get; private set; }

        /// <summary>
        /// Henter den originale statusen for cellene i utvalget.
        /// </summary>
        protected OriginalDataType[,] OriginalStates { get; private set; }

        /// <summary>
        /// Henter en enumerator for de utvalgte cellene.
        /// </summary>
        protected IEnumerable<Cell> SelectedCells
        {
            get
            {
                for (int row = this.Selection.Row; row < this.Selection.Row + this.Selection.RowCount; row++)
                {
                    for (int column = this.Selection.Column; column < this.Selection.Column + this.Selection.ColumnCount; column++)
                        yield return this.Sheet.Cells[row, column];
                }
            }
        }

        /// <summary>
        /// Produserer den originale statusen for den angitte cellen. Kalles én gang for hver av cellene
        /// ved konstruksjon av handlingen.
        /// </summary>
        /// <param name="cell"></param>
        /// <returns></returns>
        protected abstract OriginalDataType GetOriginalState(Cell cell);

        /// <summary>
        /// Setter ny status for den angitte cellen. Kalles én gang for hver av cellene
        /// ved konstruksjon av handlingen.
        /// </summary>
        /// <param name="cell"></param>
        protected abstract void SetNewState(Cell cell);

        /// <summary>
        /// Tilbakestiller den angitte cellen til den angitte originale status. Kalles én gang for hver av cellene
        /// når handlingen angres.
        /// </summary>
        /// <param name="cell"></param>
        /// <param name="originalState"></param>
        protected abstract void ResetToOriginalState(Cell cell, OriginalDataType originalState);

        /// <summary>
        /// Lagrer original status for utvalget.
        /// </summary>
        /// <returns></returns>
        protected override bool SaveUndoState()
        {
            for (int row = this.Selection.Row; row < this.Selection.Row + this.Selection.RowCount; row++)
            {
                for (int column = this.Selection.Column; column < this.Selection.Column + this.Selection.ColumnCount; column++)
                    this.OriginalStates[row - this.Selection.Row, column - this.Selection.Column] = GetOriginalState(this.Sheet.Cells[row, column]);
            }
            return true;
        }

        /// <summary>
        /// Utfører hendelsen på utvalget.
        /// </summary>
        /// <param name="sender"></param>
        /// <returns></returns>
        public override bool PerformUndoAction(object sender)
        {
            this.Sheet.AutoCalculation = false;
            try
            {
                for (int row = this.Selection.Row; row < this.Selection.Row + this.Selection.RowCount; row++)
                {
                    for (int column = this.Selection.Column; column < this.Selection.Column + this.Selection.ColumnCount; column++)
                        SetNewState(this.Sheet.Cells[row, column]);
                }
                return true;
            }
            finally
            {
                this.Sheet.AutoCalculation = true;
            }
        }

        /// <summary>
        /// Tilbakestiller utvalget til original status.
        /// </summary>
        /// <param name="sender"></param>
        /// <returns></returns>
        public override bool Undo(object sender)
        {
            this.Sheet.AutoCalculation = false;
            try
            {
                for (int row = this.Selection.Row; row < this.Selection.Row + this.Selection.RowCount; row++)
                {
                    for (int column = this.Selection.Column; column < this.Selection.Column + this.Selection.ColumnCount; column++)
                        ResetToOriginalState(this.Sheet.Cells[row, column], this.OriginalStates[row - this.Selection.Row, column - this.Selection.Column]);
                }
            }
            finally
            {
                this.Sheet.AutoCalculation = true;
            }
            return true;
        }
    }
}
